/*
 * DiamondApp.m - handles converting of key events into
 *		  appropiate Apple style keys
 * written by Chris Osborn <osborn@athena.ecs.csus.edu>
 */

#import "DiamondApp.h"
#import "apple.h"
#import <dpsclient/event.h>

#define LEFTARROW  256
#define RIGHTARROW 257
#define UPARROW    258
#define DOWNARROW  259


int AppleKeyBit, OptionKeyBit;
int AppleKey, OptionKey, ShiftKey;

static int keyLookup[93][8] = {
  {0,0,0,0,0,0,0,0}, /* No key 0x00 */
  {0,0,0,0,0,0,0,0}, /* No key 0x01 */
  {0,0,0,0,0,0,0,0}, /* No key 0x02 */
  {'\\', '|', 0x1c, 0x1c, '\\', '|', 0x1c, 0x1c},
  {']', '}', 0x1d, 0x1d, ']', '}', 0x1d, 0x1d},
  {'[', '{', 0x1b, 0x1b, '[', '{', 0x1b, 0x1b},
  {'i', 'I', 0x09, 0x09, 'I', 'I', 0x09, 0x09},
  {'o', 'O', 0x0f, 0x0f, 'O', 'O', 0x0f, 0x0f},
  {'p', 'P', 0x10, 0x10, 'P', 'P', 0x10, 0x10},
  {LEFTARROW, LEFTARROW, LEFTARROW, LEFTARROW, LEFTARROW, LEFTARROW, LEFTARROW, LEFTARROW},
  {0,0,0,0,0,0,0,0}, /* No key 0x0a */
  {'0', '0', '0', '0', '0', '0', '0', '0'},
  {'.', ',', '.', ',', '.', ',', '.', ','},
  {0x0d, 0x0d, 0x0d, 0x0d, 0x0d, 0x0d, 0x0d, 0x0d},
  {0,0,0,0,0,0,0,0}, /* No key 0x0e */
  {DOWNARROW, DOWNARROW, DOWNARROW, DOWNARROW, DOWNARROW, DOWNARROW, DOWNARROW, DOWNARROW},
  {RIGHTARROW, RIGHTARROW, RIGHTARROW, RIGHTARROW, RIGHTARROW, RIGHTARROW, RIGHTARROW, RIGHTARROW},
  {'1', '1', '1', '1', '1', '1', '1', '1'},
  {'4', '4', '4', '4', '4', '4', '4', '4'},
  {'6', '6', '6', '6', '6', '6', '6', '6'},
  {'3', '3', '3', '3', '3', '3', '3', '3'},
  {'+', '+', '+', '+', '+', '+', '+', '+'},
  {UPARROW, UPARROW, UPARROW, UPARROW, UPARROW, UPARROW, UPARROW, UPARROW},
  {'2', '2', '2', '2', '2', '2', '2', '2'},
  {'5', '5', '5', '5', '5', '5', '5', '5'},
  {0,0,0,0,0,0,0,0}, /* No key 0x19 */
  {0,0,0,0,0,0,0,0}, /* No key 0x1a */
  {127, 127, 127, 127, 127, 127, 127, 127},
  {'=', '+', '=', '+', '=', '+', '=', '+'},
  {'-', '_', 0x1f, 0x1f, '-', '_', 0x1f, 0x1f},
  {'8', '*', '8', '*', '8', '*', '8', '*'},
  {'9', '(', '9', '(', '9', '(', '9', '('},
  {'0', ')', '0', ')', '0', ')', '0', ')'},
  {'7', '7', '7', '7', '7', '7', '7', '7'},
  {'8', '8', '8', '8', '8', '8', '8', '8'},
  {'9', '9', '9', '9', '9', '9', '9', '9'},
  {'-', '-', '-', '-', '-', '-', '-', '-'},
  {'*', '*', '*', '*', '*', '*', '*', '*'},
  {'`', '~', '`', '~', '`', '~', '`', '~'},
  {'=', '|', '=', '|', '=', '|', '=', '|'},
  {'/', '\\', 0x1c, 0x1c, '/', '\\', 0x1c, 0x1c},
  {0,0,0,0,0,0,0,0}, /* No key 0x29 */
  {0x0d, 0x0d, 0x0d, 0x0d, 0x0d, 0x0d, 0x0d, 0x0d},
  {'\'', '"', '\'', '"', '\'', '"', '\'', '"'},
  {';', ':', ';', ':', ';', ':', ';', ':'},
  {'l', 'L', 0x0c, 0x0c, 'L', 'L', 0x0c, 0x0c},
  {',', '<', ',', '<', ',', '<', ',', '<'},
  {'.', '>', '.', '>', '.', '>', '.', '>'},
  {'/', '?', '/', '?', '/', '?', '/', '?'},
  {'z', 'Z', 0x1a, 0x1a, 'Z', 'Z', 0x1a, 0x1a},
  {'x', 'X', 0x18, 0x18, 'X', 'X', 0x18, 0x18},
  {'c', 'C', 0x03, 0x03, 'C', 'C', 0x03, 0x03},
  {'v', 'V', 0x16, 0x16, 'V', 'V', 0x16, 0x16},
  {'b', 'B', 0x02, 0x02, 'B', 'B', 0x02, 0x02},
  {'m', 'M', 0x0d, 0x0d, 'M', 'M', 0x0c, 0x0c},
  {'n', 'N', 0x0d, 0x0d, 'N', 'N', 0x0d, 0x0d},
  {' ', ' ', ' ', ' ', ' ', ' ', ' ', ' '},
  {'a', 'A', 0x01, 0x01, 'A', 'A', 0x01, 0x01},
  {'s', 'S', 0x13, 0x13, 'S', 'S', 0x13, 0x13},
  {'d', 'D', 0x04, 0x04, 'D', 'D', 0x04, 0x04},
  {'f', 'F', 0x06, 0x06, 'F', 'F', 0x06, 0x06},
  {'g', 'G', 0x07, 0x07, 'G', 'G', 0x07, 0x07},
  {'k', 'K', 0x0b, 0x0b, 'K', 'K', 0x0b, 0x0b},
  {'j', 'J', 0x0a, 0x0a, 'J', 'J', 0x0a, 0x0a},
  {'h', 'H', 0x08, 0x08, 'H', 'H', 0x08, 0x08},
  {0x09, 0x09, 0x09, 0x09, 0x09, 0x09, 0x09, 0x09},
  {'q', 'Q', 0x11, 0x11, 'Q', 'Q', 0x11, 0x11},
  {'w', 'W', 0x17, 0x17, 'W', 'W', 0x17, 0x17},
  {'e', 'E', 0x05, 0x05, 'E', 'E', 0x05, 0x05},
  {'r', 'R', 0x12, 0x12, 'R', 'R', 0x12, 0x12},
  {'u', 'U', 0x15, 0x15, 'U', 'U', 0x15, 0x15},
  {'y', 'Y', 0x19, 0x19, 'Y', 'Y', 0x19, 0x19},
  {'t', 'T', 0x14, 0x14, 'T', 'T', 0x14, 0x14},
  {0x1b, 0x1b, 0x1b, 0x1b, 0x1b, 0x1b, 0x1b, 0x1b},
  {'1', '!', '1', '!', '1', '!', '1', '!'},
  {'2', '@', 0x00, 0x00, '2', '@', 0x00, 0x00},
  {'3', '#', '3', '#', '3', '#', '3', '#'},
  {'4', '$', '4', '$', '4', '$', '4', '$'},
  {'7', '&', '7', '&', '7', '&', '7', '&'},
  {'6', '^', 0x1e, 0x1e, '6', '^', 0x1e, 0x1e},
  {'5', '%', '5', '%', '5', '%', '5', '%'},
};

int mapKeyToChar (int key, int flags)
/*
    I really hate to do this, but I couldn't find any documentation on
    how to turn off the Alternate mapping of keys into the Symbol font. I
    can't even make NX_ALPHASHIFTMASK | NX_SHIFTMASK downcase the key,
    because NeXT blew it and set NX_ALPHASHIFTMASK even if the shift key
    is pressed and AlphaLock isn't set.
*/
{
  int mod = ((flags & NX_SHIFTMASK) > 0) | ((flags & NX_CONTROLMASK) > 0) << 1 | ((flags & NX_ALPHASHIFTMASK) > 0) << 2;

  return keyLookup[key][mod];
}  

@implementation DiamondApp

- sendEvent:(NXEvent *) theEvent
{
  int oflags;


  oflags = theEvent->flags;
  
  switch (theEvent->type) {
  case NX_KEYDOWN:
  case NX_KEYUP:
  case NX_FLAGSCHANGED:
    AppleKey = (theEvent->flags & AppleKeyBit) != 0;
    OptionKey = (theEvent->flags & OptionKeyBit) != 0;
    ShiftKey = (theEvent->flags & NX_SHIFTMASK) != 0;
    
    theEvent->flags &= (~OptionKeyBit) & (~AppleKeyBit);
    if ((theEvent->flags & NX_ALTERNATEMASK) && !(theEvent->flags & (NX_NEXTLALTKEYMASK | NX_NEXTRALTKEYMASK)))
      theEvent->flags &= ~NX_ALTERNATEMASK;
    if ((theEvent->flags & NX_COMMANDMASK) && !(theEvent->flags & (NX_NEXTLCMDKEYMASK | NX_NEXTRCMDKEYMASK)))
      theEvent->flags &= ~NX_COMMANDMASK;

    if (!(theEvent->flags & NX_ALTERNATEMASK) && (oflags & NX_ALTERNATEMASK)) {
      theEvent->data.key.charCode = mapKeyToChar (theEvent->data.key.keyCode, theEvent->flags);
      theEvent->data.key.charSet = NX_ASCIISET;
    }
    
    break;
  }

  [super sendEvent:theEvent];
  
  return self;
}

@end


